/*
 * Created on Apr 13, 2005
 *
 */
package com.sunlabs.archsim.fleet.components.hornAndFunnelFabric;

// import java.util.Random;

import com.sunlabs.archsim.base.common.Parameters;
import com.sunlabs.archsim.base.hierarchy.CompositeEntity;
import com.sunlabs.archsim.base.simulation.Delay;
import com.sunlabs.archsim.fleet.common.FleetGlobal;
import com.sunlabs.archsim.fleet.common.Payload;
import com.sunlabs.archsim.gasp.Action;
import com.sunlabs.archsim.gasp.Component;
import com.sunlabs.archsim.gasp.Function;
import com.sunlabs.archsim.gasp.Terminal;
import com.sunlabs.archsim.gasp.terminals.InputTerminal;
import com.sunlabs.archsim.gasp.terminals.OutputTerminal;


/**
 *
 * Copyright  2005 Sun Microsystems Inc. All rights reserved. Use is subject to license terms.
 * 
 * @author ib27688
 *
 * Test instruction generator.
 * Moves data from each source to each destination.
 * 0->0, 0->1, ... 0->n, 1->0, 1->1, ... 1->, ... n->0, n->1, ... n->n 
 */
public class TestInstrGen extends Component {
	
	TestInstrGenSource src;
	
	OutputTerminal out;
	InputTerminal ackOut;
	
	
	
	public TestInstrGen(String name, Parameters params) {
		super(name, null);

		// process the parameters
		int maxParam= 0;
		if (params != null) {
			for (int i= 0; i<params.size(); i++) {
				String pName= params.getName(i);
				String sValue= params.getValue(i);
				
				if (pName.equals(FleetGlobal.PARAMETER) || pName.equals("max")) {
					try {
						maxParam= Integer.parseInt(sValue);
					}
					catch (Exception e) {
						director.error("Component: " + name + ", id= " + getID() 
								+ ". Bogus value used for "
								+ "max/" + FleetGlobal.PARAMETER
								+ ": " + sValue
								+ ". Value is IGNORED, default value of "
								+ maxParam
								+ " is used.");						
					}
				}
				else {
					director.error("Component: " + name + ", id= " + getID() 
							+ ": Attribute " + pName + " with value " + sValue
							+ " is ignored");					
				}
				
			}
		} // if params != null	
		
		buildTestInstrGenerator(maxParam);
	}

	
	public TestInstrGen(String name, int n, Parameters params) {
		super(name, null);
		
		// process the parameters
		if (params != null) {
			for (int i= 0; i<params.size(); i++) {
				String pName= params.getName(i);
				String sValue= params.getValue(i);

				director.error("Component: " + name + ", id= " + getID() 
						+ ": Attribute " + pName + " with value " + sValue
						+ " is ignored");					
				
			}
		} // if params != null	
		
		buildTestInstrGenerator(n);
		
	} // TestInstrGen constructor

	protected void buildTestInstrGenerator(int n) {
		src= new TestInstrGenSource(FleetGlobal.SOURCE, n, this);
		
		out= new OutputTerminal(FleetGlobal.OUT, this);
		ackOut= new InputTerminal(FleetGlobal.ACK_OUT, this);
		
		src.attach(out);
		src.attach(ackOut);		
	}
	
	public OutputTerminal getOut() {
		return out;
	}
	
	public InputTerminal getAckOut() {
		return ackOut;
	}
	
	public Terminal getTerminal(String name) {
		if (name.equals(out.getBaseName()) 
				|| name.equals(out.getBaseAlias())) 
			return out;
		else if (name.equals(ackOut.getBaseName())
				|| name.equals(ackOut.getBaseAlias()))
			return ackOut;
		else return null;
	}
	
} // TestGen

class TestInstrGenSource extends Action {
	
	private Delay sourceDelay= FleetGlobal.INSTRUCTION_SOURCE_DELAY;

	Function out;
	
	public TestInstrGenSource(String name, int max, CompositeEntity parent) {
		super(name, parent);
		out= new GenerateData(max);
		outputFunctions.put(FleetGlobal.OUT, out);
	} // FifoAction
	
	public void setDelay(Delay d) {
		sourceDelay= (Delay)d.clone();
		out.setDelay(sourceDelay);
	}
	
	/** Generate new data items */
	private class GenerateData extends Function {

		private int max;
		
		private int nextSrc;
		private int nextDest;
		
		// private Random rndGen;
		
		public GenerateData(int max) {
			super("GenerateData");
			this.max= max;
			// rndGen= new Random();
			nextSrc= 0; // rndGen.nextInt(this.max); // 0;
			nextDest= 0; // rndGen.nextInt(this.max); // 0;
			functionDelay= sourceDelay;
		}
		
		public Object value() {
			Payload payload= new Payload();
			payload.source= nextSrc;
			payload.destination= nextDest;
			
			// prepare the sources and destinations for the next trip around
			nextDest++;
			if (nextDest >= max) {
				nextDest= 0;
				nextSrc++;
				if (nextSrc >= max) nextSrc= 0;
			}
			
			
/*			nextSrc= rndGen.nextInt(this.max);
			nextDest= rndGen.nextInt(this.max); 
*/
			
			if (FleetGlobal.LOG_INSTRUCTION_SOURCE) {
				director.journal("Source " + TestInstrGenSource.this.getName() 
						+ ", id= " + TestInstrGenSource.this.getID()
						+ ", generated " + payload);
			}
			return payload;	
		} // value
	} // PassData

} // TestGenSource
