/*
 * Copyright 2005 Sun Microsystems Inc. All rights reserved. Use is subject to license terms.
 *  
 */
package com.sunlabs.archsim.fleet.ships.bitbucket;

import java.util.List;
import java.util.LinkedList;

import com.sunlabs.archsim.base.hierarchy.Entities;
import com.sunlabs.archsim.base.hierarchy.CompositeEntity;
import com.sunlabs.archsim.engine.Director;
import com.sunlabs.archsim.fleet.common.Data;
import com.sunlabs.archsim.fleet.common.Payload;
import com.sunlabs.archsim.fleet.ships.testing.PayloadSource;
import com.sunlabs.archsim.runner.OperationLoader;
import com.sunlabs.archsim.runner.Runner;

import junit.framework.TestCase;
import junit.textui.TestRunner;

public class BitBucketTest extends TestCase {
	
	Director director= Director.getInstance();
	
	private BitBucket bitBucket;
	
	private LinkedList<Object> referenceData= new LinkedList<Object>();
	
	// used to run tests "manually
	public static void main(String []rgs) {
        // run the tests
        TestRunner.run(BitBucketTest.class);		
	}
	
	public void setUp() {
		// add data items
		for (int i= 0; i<10; i++) {
			Data d= new Data(i);
			Payload p= new Payload();
			p.data= d;
			PayloadSource.add("DataSource", d);
			referenceData.add(p);
		}
		

	} // setUp
	
	public void tearDown() {
		director.masterClear();
	}
	

	// first do the self check
	public void testSelfCheck() {
		Object tst=Entities.find("BitBucketTest");
		CompositeEntity testGroup= (CompositeEntity)tst;
		assert(testGroup.selfCheck());		
	}
	
	// now test the operation of a BitBucket
	public void testOperation() {
		OperationLoader ldr= new OperationLoader();
		List ops= ldr.load("src/" 
				+ BitBucketTest.class.getPackage().getName().replace('.', '/') 
				+ "/BitBucketTestScript.xml");
		
		Runner r= new Runner();
		r.perform(ops);
		
		// get the bit bucket
		bitBucket= (BitBucket)(Entities.find("BitBucket"));
		
		assert(referenceData.equals(bitBucket.receivedData));
		
	} // operationTest
	
} // bitBucketTest
